<?php

namespace App\Http\Controllers\API\V1;

use DB;
use App\Models\User;
use Illuminate\Support\Str;
use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use DateTime;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Crypt;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;
use Tymon\JWTAuth\Facades\JWTAuth;
use Tymon\JWTAuth\Token;
use Carbon\Carbon;

class AuthController extends Controller
{
    function register(Request $request)
    {

        try {
            $data = $request->validate([
                "name" => "required",
                "email" => "required|email",
                "password" => "required",
            ]);
            $data['password'] = Hash::make($data['password']);
            //$user = User::create($data);



            // Check if the email already exists
            $existingClient = DB::connection('login')
                ->table('dbo.users')
                ->where('email', $data["email"])
                ->first();

            if (!$existingClient) {
                // Email doesn't exist, so insert the new client
                DB::connection('login')
                    ->table('dbo.users')
                    ->insert([
                        "email" => $data["email"],
                        "name" => $data["name"],
                        "password" => $data["password"],
                        "created_at" => date("Y-m-d H:i:s")
                    ]);

                // User registerd so return the login
                return $this->login($request);
                //return response()->json(['status' => '1', 'message' => "User created!"]);
            } else {
                // Email already exists
                return response()->json(['status' => 'error', 'message' => "User already exist!"]);
            }



            //return response()->json(['status' => 'success', 'message' => 'user created','user' => $user], 200);
        } catch (\Exception $e) {
            return response()->json(['status' => 'error', 'message' => $e->getMessage()]);
        }
    }

    function login(Request $request)
    {

        $request->validate([
            "email" => "required",
            "password" => "required",
        ]);

        // Extract email and password from the request
        $email = $request->input('email');
        $password = $request->input('password');

        return $this->loginReturnData($email, $password, $request);
    }


    function loginReturnData($email, $password, Request $request)
    {

        try {

            $credentials = ['email' => $email, 'password' => $password];

            if (Auth::attempt($credentials)) {
                $user = Auth::user();


                // Prepare the payload
                $payload = [
                    'user_agent' => $request->userAgent(),
                    'ip' => $request->ip(),
                    'tokenGenTime' => Carbon::now()->toDateTimeString(),
                    'signoutOnIpChange' => false,
                    'ps' => $password, // Include the 'ps' property in the payload
                    'exp' => now()->addDays(20)->timestamp
                ];
                $user->setCustomClaims($payload);


                $token = JWTAuth::fromUser($user, $payload);
                $userId = $user->id;
                $userClients = $this->getUserClients($userId, $password);
                $userData = Auth::user()->toArray();
                unset($userData['password']);
                unset($userData['remember_token']);
                unset($userData['pin_code']);


                // ---------- Get last connect hash ------------
                $LastConnectHash = "";
                // Get the LastConnectHash
                $settingsRow = DB::connection('login')->table('ClientUserSettings')
                    ->where('UserID', '=', (string) $userId)
                    ->where('SettingID', '=', 'LastConnectedHash')
                    ->first();
                if ($settingsRow !== null) {
                    // Data exists, you can access properties
                    $LastConnectHash = $settingsRow->SettingValue;
                } 

                // ---------------------------------------------


                return response()->json(['status' => '1', 'message' => 'login successfully', 'lastConnectHash' => $LastConnectHash, 'access_token' => $token, 'user' => $userData, 'userClients' => $userClients], 200);
            }
            return response()->json(['status' => 'unauthenticated', 'message' => 'Invalid credentials'], 401);
        } catch (\Exception $e) {
            return response()->json(['status' => 'error', 'message' => $e->getMessage()]);
        }
    }

    function getUserClients($userId, $pass = null)
    {

        $clients = DB::connection('login')->table('Clients')
            ->select('Clients.*')
            ->join('ClientUserMapping', 'Clients.id', '=', 'ClientUserMapping.Client_ID')
            ->where('ClientUserMapping.UserID', (string) $userId)
            ->get();
        // dd($clients);
        $clientData = [];
        foreach ($clients as $key => $client) {
            // $startDateObj = new DateTime($client->ServiceStartDate);
            $endDateObj = new DateTime($client->ServiceEndDate);
            $allowedGracePeriodDays = $client->AllowedGracePeriodDays;
            // Get the current date
            $currentDate = new DateTime();

            // Compare the current date with the end date
            if ($currentDate > $endDateObj) {
                // Service has expired
                $lincenseExpired = true;
                $daysLeft = 0;
            } else {
                // Service is still active
                $lincenseExpired = false;
                // Calculate the difference between the current date and the end date
                $interval = $currentDate->diff($endDateObj);
                // Get the total number of days until the end date
                $totalDays = $interval->days;

                // Calculate the number of days left considering the grace period
                $daysLeft = max(0, $totalDays - $allowedGracePeriodDays);
            }
            $data = [
                'clientID' => $client->Client_ID,
                'ClientName' => $client->ClientName,
                // 'type' => 'live',

                'SuspendConnection' => $client->SuspendConnection,
                'MaxUserConnections' => $client->MaxUserConnections,
                'CurrentUserConnections' => $client->CurrentUserConnections,
                'ServiceEndDate' => $client->ServiceEndDate,
                'AllowedGracePeriodDays' => $client->AllowedGracePeriodDays,
                'gracePeriodsDayLeft' => $daysLeft,
                'licenseExpired' => $lincenseExpired,
                "DB_IdentificationName" => "Live (dynamo)",
            ];
            $licenseToken = [
                'SuspendConnection' => $client->SuspendConnection,
                'gracePeriodsDayLeft' => $daysLeft,
                'licenseExpired' => $lincenseExpired,
                'rowId' => $client->Client_ID
            ];

            $data['ConnectToken'] = Crypt::encrypt([
                "Db" => [
                    'DB_ConnectionURL' => $client->DB_ConnectionURL,
                    'DB_UserID' => $client->DB_UserID,
                    'DB_Password' => $client->DB_Password,
                    'DB_Type' => $client->DB_Type,
                    'DB_Driver' => $client->DB_Driver,
                    'DB_Dialect' => $client->DB_Dialect,
                    'DB_ShowLog' => $client->DB_ShowLog,
                ],
                'LicenseToken' => $licenseToken,
                'password' => $pass
            ]);
            $data['connectHash'] = $this->connectTokenHash('live',$client->Client_ID , $client->DB_ConnectionURL);
            //$data['testingConnectHash'] = ["type"=> 'live', "clientId"=> $client->Id, 'dbUrl'=> $client->DB_ConnectionURL];
            $data['dbRowId'] = $client->Id;
            $data['type'] = 'live';
            $data['archiveClients'] = $this->getArchiveClients($client->Id, $licenseToken, $pass);
            array_push($clientData, $data);
        }
        return $clientData;
    }

    function getArchiveClients($clientId, $licenseToken, $pass = null)
    {
        $clients = DB::connection('login')->table('ClientArchiveDBs')
            ->where('Client_ID', (string) $clientId)
            ->get();
        $archiveCompaniesData = [];

        foreach ($clients as $key => $client) {
            $data = [
                'Client_ID' => $clientId,
                'DB_IdentificationName' => "Archive : " . $client->DB_IdentificationName,
                "ConnectToken" => Crypt::encrypt([
                    "Db" => [
                        'DB_ConnectionURL' => $client->DB_ConnectionURL,
                        'DB_UserID' => $client->DB_UserID,
                        'DB_Password' => $client->DB_Password,
                    ],
                    "LicenseToken" => $licenseToken,
                    "password" => $pass,
                ]),
                "connectHash" => Crypt::encrypt('archive-' . $client->Id . '-' . $client->DB_ConnectionURL),
                'dbRowId' => $client->Id,
                'type' => 'archive',
            ];
            array_push($archiveCompaniesData, $data);
        }
        return $archiveCompaniesData;
    }


    public function refreshToken(Request $request)
    {

        $userDat = Auth::user();

        $userJwtPayload = $request->attributes->get('userJwtPayload');

        $email = $userDat->email;
        // Get the password
        $password = $userJwtPayload["ps"];

        return $this->loginReturnData($email, $password, $request);
    }

    public function refreshTokenPinCode(Request $request)
    {

        $request->validate([
            "pincode" => "required",
        ]);


        try {


            // Now you can parse the token
            $token = JWTAuth::parseToken();

            // Authenticate the user
            $user = $token->authenticate();

            // if the pincode not matched
            $user_pin = (string) $user->pin_code;
            $req_pin = (string) $request->input('pincode');

            $userJwtPayload = $token->getPayload();

            $email = $user->email;

            // Get the password
            $password = $userJwtPayload["ps"];

            // If pin code not matched
            if ($user_pin !== $req_pin) {
                return response()->json(['status' => '0', 'message' => 'Pin code is invalid', 'retrysLeft' => 4], 200);
            }

            // We are here it means all good so get login
            return $this->loginReturnData($email, $password, $request);
        } catch (\Exception $e) {
            return response()->json(['status' => 'error', 'message' => $e->getMessage()]);
        }
    }

    public function verifyLogin(Request $request)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $updated = [];
        $validator = Validator::make($request->all(), [
            'ConnectToken' => 'required',
            'dbRowId' => 'required',
            'clientID' => 'required',
            'type' => 'required',
            'rememberMe' => 'required'
        ]);

        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->errors()->all();
            return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => []], 422);
        }
        try {

            $usrObj = Auth::user();

            $tokenData = Crypt::decrypt($request->ConnectToken);
            $dbRowId = $request->dbRowId;
            $clientID = $request->clientID;
            $type = $request->type;

            $connectionString = $tokenData['Db']['DB_ConnectionURL'];
            $usr_password = $tokenData['password'];
            $usr_email = $usrObj->email;
            $usr_mainId = $usrObj->id;
            // $userId = "sa";
            $userId = $tokenData['Db']['DB_UserID'];
            // $password = "SA++2024**";
            $password = $tokenData['Db']['DB_Password'];
            // Define regular expressions to match URL and database name
            $urlPattern = '/jdbc:sqlserver:\/\/(.*?):(\d+);/';
            $databaseNamePattern = '/databaseName=(.*?)(;|$)/';

            // Match URL and port number
            preg_match($urlPattern, $connectionString, $urlMatches);
            $hostname = $urlMatches[1]; // Extract URL
            $port = $urlMatches[2]; // Extract port number

            // Match database name
            preg_match($databaseNamePattern, $connectionString, $databaseNameMatches);
            $databaseName = $databaseNameMatches[1]; // Extract database name

            // Your custom database configuration
            $customConfig = [
                'driver' => 'sqlsrv',
                'host' => $hostname,
                'port' => $port,
                'database' => $databaseName,
                'username' => $userId,
                'password' => $password,
                // Add other configuration parameters as needed
            ];

            // Set your custom configuration temporarily
            DB::purge('sqlsrv'); // Make sure to replace 'your_connection_name' with the actual connection name
            config(["database.connections.sqlsrv" => $customConfig]);


            // Process the result or perform other database operations
            $encryptedPassword = $this->encryptPassword($usr_password);
            $user = DB::connection('sqlsrv')->table('Users')
                ->where('usr_email', $usr_email)
                ->where('usr_password', $encryptedPassword)->first();
            if (empty($user)) {
                $data = [];
                $status = "0";
                $message = "User not found";
                $errors = [
                    'error' => "User not found"
                ];
                return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => $data], 422);
            } else {
                $userCompanies = DB::connection('sqlsrv')->table('UserCompany')
                    ->where('ucp_userid', $user->usr_userid)->get()->pluck('ucp_compcode');
                $companiesData = DB::connection('sqlsrv')->table('Company')
                    ->select('cmp_compcode', 'cmp_compname', 'cmp_comptype')->whereIn('cmp_compcode', $userCompanies)->get()->toArray();
                $data = [
                    'userData' => $user,
                    'companies' => $companiesData,
                ];
                $updated = $this->updateSetting($request->ConnectToken, $dbRowId, $clientID, $type, $usr_mainId, $request->rememberMe);
            }
            $status   = "1";
            return response()->json(['status' => $status, 'rememberMe'=> $request->rememberMe, 'error' => (object)$errors, 'message' => $message, 'oData' => $data, 'updated' => $updated], 200);
        } catch (\Throwable $th) {
            $errors = [
                'error' => $th->getMessage()
            ];
            return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $th->getMessage(), 'oData' => []], 422);
        }
    }

    public function clientConnected(REQUEST $request)
    {

        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        $validator = Validator::make($request->all(), [
            'clientId'           => 'required',
            'userName'          => 'required',
            'connectHash'       => 'required'
        ]);

        // If validation failed
        if ($validator->fails()) {
            $status = "0";
            $message = "Validation error occured";
            $errors = $validator->messages();

            // Return response
            return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
        }


        // We are here it means all fields validated


        $client_id = $request->clientId;
        $user_id = strtoupper($request->userName);
        $connectHash = $request->connectHash;


        // Update/Create lastconnectDate
        DB::connection('login')
            ->table('ClientUserSettings')
            ->updateOrCreate(
                [
                    [
                        'client_id' => $client_id,
                        'UserId' => $user_id,
                        'SettingId' => "LastConnectedDate",
                    ]
                ],
                [

                    'SettingValue' => Carbon::now()->format('d-M-Y') // Get current date
                ]
            );


        // Update/Create lastConnectHash
        DB::connection('login')
            ->table('ClientUserSettings')
            ->updateOrCreate(
                [
                    [
                        'client_id' => $client_id,
                        'UserId' => $user_id,
                        'SettingId' => "LastConnectHash",
                    ]
                ],
                [

                    'SettingValue' => $connectHash
                ]
            );

        $userPayload = $request->userJwtPayload;

        return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$userPayload], 200);
    }

    public function updateSetting($connectToken, $dbRowId, $clientId, $type, $usr_userid, $rememberMe)
    {
        $status   = "0";
        $message  = "";
        $o_data   = [];
        $errors   = [];
        try {
            $tokenData = Crypt::decrypt($connectToken);
            $dbRowId = $dbRowId;
            $clientId = $clientId;
            $type = $type;
            // connection string work 
            // $connectionString = "jdbc:sqlserver://217.165.28.252:1433;databaseName=iBridgeERP-Demo";

            $connectionString = $tokenData['Db']['DB_ConnectionURL'];
            // $userId = "sa";
            $userId = $tokenData['Db']['DB_UserID'];
            // Process the result or perform other database operations


            $authUserId = strtoupper($usr_userid);

            // Update/insert the last connected date
            DB::connection('login')
                ->table('ClientUserSettings')
                ->updateOrInsert(
                    [
                        'client_id' => $clientId,
                        'UserId' =>  $authUserId,
                        'SettingId' => "LastConnectedDate",
                    ],
                    [

                        'SettingValue' => date('d-M-Y') // Get current date
                    ]
                );


            $connectHash = $this->connectTokenHash($type, $clientId, $connectionString);
            $testing_connectData = ["type"=> $type, "clientId"=> $clientId, 'dbUrl'=> $connectionString];
        

            //$connectHash = substr($connectHash, 0, 200); // @todo remove this after we increase the column allowed characters to 500


            // Update/Create lastConnectHash if $rememberMe is true
            if ($rememberMe) {

                DB::connection('login')
                ->table('ClientUserSettings')
                ->updateOrInsert(

                    [
                        //'client_id' => '',
                        'UserId' => $authUserId,
                        'SettingId' => "LastConnectedHash",
                    ],
                    [

                        'SettingValue' => $connectHash
                    ]
                );

            }
            


            // $clientUserSetting = DB::connection('login')->table('ClientUserSettings')
            //     ->where('Client_ID', $clientId)->first();
            // $authUserId = $usr_userid;
            // if (empty($clientUserSetting)) {
            //     DB::connection('login')->table('ClientUserSettings')
            //         ->insert([
            //             [
            //                 'Client_ID' => $clientId,
            //                 'UserId' => $authUserId,
            //                 'SettingID' => 'LastConnectedURL',
            //                 'SettingValue' => Crypt::encrypt($type . '-' . $clientId . '-' . $connectionString),
            //             ],
            //             [
            //                 'Client_ID' => $clientId,
            //                 'UserId' => $authUserId,
            //                 'SettingID' => 'LastConnectedDate',
            //                 'SettingValue' => date('d-M-Y'),
            //             ],
            //         ]);
            // } else {
            //     DB::connection('login')->table('ClientUserSettings')
            //         ->where('Client_ID', $clientId)->where('SettingID', 'LastConnectedURL')->where("UserId", $authUserId)
            //         ->update(['SettingValue' => Crypt::encrypt($type . '-' . $clientId . '-' . $connectionString)]);

            //     DB::connection('login')->table('ClientUserSettings')
            //         ->where('Client_ID', $clientId)->where('SettingID', 'LastConnectedDate')->where("UserId", $authUserId)
            //         ->update(['SettingValue' => date('d-M-Y')]);
            // }
            $status   = "1";
            return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => []], 200);
        } catch (\Throwable $th) {
            $errors = [
                'error' => $th->getMessage()
            ];
            return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $th->getMessage(), 'oData' => []], 422);
        }
    }

    public function encryptPassword($password)
    {
        $encryptedPassword = '';
        $len = strlen($password);

        for ($i = 0; $i < $len; $i++) {
            $char = $password[$i];
            $encryptedPassword .= dechex(ord($char));
        }

        return $encryptedPassword;
    }

    public function decryptPassword($encryptedPassword)
    {
        $decryptedPassword = '';
        $len = strlen($encryptedPassword);

        for ($i = 0; $i < $len; $i += 2) {
            $hexChar = substr($encryptedPassword, $i, 2);
            $decryptedPassword .= chr(hexdec($hexChar));
        }

        return $decryptedPassword;
    }

    private function connectTokenHash($type, $client_id, $connectionUrl) {
        return md5($type.'-' . $client_id . '-' . $connectionUrl);
    }
}
