<?php

namespace App\Http\Controllers\master;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\CountryModel;
use App\Models\LookupTableData;
use App\Models\CommodityOriginModel;
use App\Models\CountryVatRegionModel;
use App\Models\Destination;
use App\Models\Address;
use App\Models\PartyModel;
use App\Models\BankDetails;
use App\Models\DeliveryPlanAddress;
use App\Models\StoreLocation;
use App\Models\TradeModel;
use App\Services\ReportService;
use Validator;
use DB;
use Illuminate\Support\Facades\Auth;

class CountryController extends Controller
{
  //
  private function validateAccesToken($access_token)
  {
    return ['id' => 1, 'name' => 'ADMIN'];
  }
  public function index(REQUEST $request)
  {
    $status   = "0";
    $message  = "";
    $o_data   = [];
    $errors   = [];
    $validator = Validator::make($request->all(), [
      // 'access_token' => 'required'/
    ]);

    if ($validator->fails()) {
      $status = "0";
      $message = "Validation error occured";
      $errors = $validator->messages();
    } else {
      $page    = $request->page ?? 1;
      $limit   = $request->limit ?? 10;
      $offset  = ($page - 1) * $limit;
      $list    =  CountryModel::take($limit)->skip($offset)->get();
      if ($list->count() > 0) {
        $o_data['list'] = convert_all_elements_to_string($list->toArray());
        $status = "1";
        $message = "data fetched successfully";
      } else {
        $message = "no data to show for your request";
      }
    }

    return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
  }
  public function show(REQUEST $request)
  {
    $status   = "0";
    $message  = "";
    $o_data   = [];
    $errors   = [];
    $validator = Validator::make($request->all(), [
      'id'           => 'required'
    ]);

    if ($validator->fails()) {
      $status = "0";
      $message = "Validation error occured";
      $errors = $validator->messages();
    } else {
      $page    = $request->page ?? 1;
      $limit   = $request->limit ?? 10;
      $offset  = ($page - 1) * $limit;
      $list    =  CountryModel::where(['CTY_IdCountry' => $request->id])->get();
      if ($list->count() > 0) {
        $o_data = convert_all_elements_to_string($list->first()->toArray());
        $status = "1";
        $message = "data fetched successfully";
      } else {
        $message = "no data to show for your request";
      }
    }

    return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
  }
  public function create(Request $request)
  {
    // return ($request->access_token);
    $status = "0";
    $message = "";
    $o_data = [];
    $errors = [];
    $validator = Validator::make($request->all(), [
      'country_code' => 'required|max:5',
      'country_name' => 'required|max:255',
      'trading_area' => 'max:20|nullable|exists:CONT_LookupTableData,LKD_Code'
    ]);

    if ($validator->fails()) {
      $status = "0";
      $message = "Validation error occurred";
      $errors = $validator->messages();
    } else {

      $user = Auth::user();
      // return $user;
      $existingCountryCode = CountryModel::where('CTY_IdCountry', strtoupper($request->country_code))->exists();
      $existingCountryName = CountryModel::where('CTY_NmeCountry', $request->country_name)->exists();

      if ($existingCountryCode) {
        $errors['country_code'] = ["Country code already exists"];
      } elseif ($existingCountryName) {
        $errors['country_name'] = ["Country name already exists"];
      } else {
        DB::beginTransaction();
        try {
          $country = new CountryModel();
          $country->CTY_IdCountry = strtoupper($request->country_code);
          $country->CTY_NmeCountry = $request->country_name;
          $country->CTY_DtLastUpdated = now(); // Use Laravel's now() helper
          $country->CTY_NmeUserLastUpdated = $user['name'];
          $country->CTY_IsPartOfEU = $request->is_part_of_eu == 1 ? 'Y' : 'N';
          $country->CTY_IdTradingArea = $request->trading_area ?? '';
          $country->save();
          DB::commit();
          $status = "1";
          $message = "Record saved successfully";
        } catch (\Exception $e) {
          DB::rollback();
          $message = "Transaction Failed: " . $e->getMessage();
        }
      }
    }

    return response()->json(['status' => $status, 'errors' => $errors, 'message' => $message, 'oData' => $o_data], 200);
  }

  public function get_trade(REQUEST $request)
  {
    $status = "0";
    $message = "";
    $o_data = [];
    $errors = [];
    $validator = Validator::make($request->all(), [
      // 'access_token' => 'required',
      // 'id'           => 'required'
    ]);

    if ($validator->fails()) {
      $status = "0";
      $message = "Validation error occurred";
      $errors = $validator->messages();
    } else {
      $list = TradeModel::where('LKD_TableID', 'TRADE-AREA')->get();

      if ($list->count() > 0) {
        $o_data = $list->toArray();
        $status = "1";
        $message = "Data fetched successfully";
      } else {
        $message = "No data to show for your request";
      }
    }

    return response()->json(['status' => $status, 'error' => (object) $errors, 'message' => $message, 'oData' => $o_data], 200);
  }


  // Adjust the response structure based on your API


  public function update(REQUEST $request)
  {
    $status   = "0";
    $message  = "";
    $o_data   = [];
    $errors   = [];
    $validator = Validator::make($request->all(), [
      'country_code'           => 'required|max:5',
      'country_name'           => 'required|max:255',
      'is_part_of_eu'          => 'required|max:1',
      'trading_area'           => 'max:20|nullable|exists:CONT_LookupTableData,LKD_Code'
    ]);

    if ($validator->fails()) {
      $status = "0";
      $message = "Validation error occured";
      $errors = $validator->messages();
    } else {
      $user = Auth::user();

      // Check if another country with the new CTY_IdCountry already exists
      $existingCountryName = CountryModel::where('CTY_IdCountry', '!=', strtoupper($request->country_code))
        ->where('CTY_NmeCountry', $request->country_name)
        ->exists();

      if ($existingCountryName) {
        $errors['country_name'] = ["Country name already exists"];
      } else {
        DB::beginTransaction();
        try {
          CountryModel::where(['CTY_IdCountry' => strtoupper($request->country_code)])->update(
            [
              'CTY_NmeCountry' => $request->country_name,
              'CTY_DtLastUpdated' => date('Y-m-d H:i:s'),
              'CTY_NmeUserLastUpdated' => $user['name'],
              'CTY_IsPartOfEU' => ($request->is_part_of_eu == 1) ? 'Y' : 'N',
              'CTY_IdTradingArea' => $request->trading_area
            ]
          );
          DB::commit();
          $status = "1";
          $message = "Record updated successfully";
        } catch (\Exception $e) {
          DB::rollback();
          $message = "Transaction Faild: " . $e->getMessage();
        }
      }
    }

    return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
  }

  public function delete(REQUEST $request)
  {
    $status   = "0";
    $message  = "";
    $o_data   = [];
    $errors   = [];
    $validator = Validator::make($request->all(), [
      'country_code'           => 'required|max:5'
    ]);
    try {
      if ($validator->fails()) {
        $status = "0";
        $message = "Validation error occured";
        $errors = $validator->messages();
      } else {
        $check = CountryModel::where(['CTY_IdCountry' => strtoupper($request->country_code)])->get();
        if ($check->count() > 0) {
          DB::beginTransaction();
          try {
            $check_vat_region = CountryVatRegionModel::where(['VAT_IdCountry' => $request->country_code])->get()->count();
            if ($check_vat_region > 0) {
              $message = "You cant delete this record its associated with other record";
            } else {
              $check_commodity_origin = CommodityOriginModel::where(['OR_IdCountry' => $request->country_code])->get()->count();
              if ($check_commodity_origin > 0) {
                $message = "You cant delete this record its associated with other record";
              } else {
                $check_party = PartyModel::where(['PTY_IdBuyerBankCountry' => $request->country_code])->get()->count();
                if ($check_party > 0) {
                  $message = "You cant delete this record its associated with other record";
                } else {
                  $check_destination = Destination::where(['DST_IdCountry' => $request->country_code])->get()->count();
                  if ($check_destination > 0) {
                    $message = "You cant delete this record its associated with other record";
                  } else {
                    $check_address = Address::where(['adr_country' => $request->country_code])->get()->count();
                    if ($check_address > 0) {
                      $message = "You cant delete this record its associated with other record";
                    } else {
                      $check_party_seller = PartyModel::where(['PTY_IdSellerBankCountry' => $request->country_code])->get()->count();
                      if ($check_party_seller > 0) {
                        $message = "You cant delete this record its associated with other record";
                      } else {
                        $checkbank_details = BankDetails::where(['BDS_IdCountry' => $request->country_code])->get()->count();
                        if ($checkbank_details > 0) {
                          $message = "You cant delete this record its associated with other record";
                        } else {
                          $checkdelivery_plan = DeliveryPlanAddress::where(['DPA_AddressCountry' => $request->country_code])->get()->count();
                          if ($checkdelivery_plan > 0) {
                            $message = "You cant delete this record its associated with other record";
                          } else {
                            $check_store_loc = StoreLocation::where(['STL_IdCountry' => $request->country_code])->get()->count();
                            if ($check_store_loc > 0) {
                              $message = "You cant delete this record its associated with other record";
                            } else {
                              CountryModel::where(['CTY_IdCountry' => strtoupper($request->country_code)])->delete();
                              DB::commit();
                              $status = "1";
                              $message = "Country deleted successfully";
                            }
                          }
                        }
                      }
                    }
                  }
                }
              }
            }
          } catch (\Exception $e) {
            DB::rollback();
            $message = "Transaction Faild: " . $e->getMessage();
          }
        } else {
          $message = "record not found";
        }
      }
    } catch (\Exception $e) {
      $status = "0";
      $message = $e->getMessage();
      $errors = [
        'error' => $e->getMessage()
      ];
    }

    return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
  }


  //Data datatable
  public function datatable_list(REQUEST $request)
  {
    $status   = "0";
    $message  = "";
    $o_data   = [];
    $errors   = [];
    try {
      return $this->get_country_list($request->all());
    } catch (\Exception $e) {
      $status = "0";
      $message = $e->getMessage();
      $errors = [
        'error' => $e->getMessage()
      ];
    }

    return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => (object)$o_data], 200);
  }
  private function _get_student_list($searchQuery = '', $needSql = FALSE, $filter = [])
  {
    //$list = CountryModel::wher('CTY_IdCountry','asc');
    $list = CountryModel::select('*');
    if ($filter) {
      $list = $list->where($filter);
    }
    if ($searchQuery != '') {
      $list = $list->whereRaw($searchQuery);
    }
    if ($needSql) {
      return $list = $list->toSql();
    } else {
      return $list;
    }
  }
  public function get_country_list($postData = null)
  {

    $response = array();



    ## Read value

    $draw = $postData['draw'];

    $start = $postData['start'];

    $rowperpage = $postData['length']; // Rows display per page

    $columnIndex = $postData['order'][0]['column']; // Column index

    $columnName = $postData['columns'][$columnIndex]['data']; // Column name

    $columnSortOrder = $postData['order'][0]['dir']; // asc or desc

    $searchValue = $postData['search']['value']; // Search value







    $filter = [];



    ## Search

    $searchQuery = "";

    if ($searchValue != '') {

      $searchQuery = " (CTY_IdCountry like '%" . $searchValue . "%' or CTY_NmeCountry like '%" . $searchValue . "%' or CTY_IdTradingArea like '%" . $searchValue . "%'  ) ";
    }



    ## Total number of records without filtering

    $list = $this->_get_student_list('', FALSE, $filter);
    $totalRecords = $list->get()->count();



    ## Total number of record with filtering

    $list = $this->_get_student_list($searchQuery, FALSE, $filter);
    $totalRecordwithFilter = $list->get()->count();

    ## Fetch records

    $list = $this->_get_student_list($searchQuery, FALSE, $filter);
    $list = $list->orderBy($columnName, $columnSortOrder);
    if ($rowperpage != 'all') {
      $list = $list->skip($start)->take($rowperpage);
    }
    $records = $list->get();



    $data = array();
    // work related to pdf.
    $exportPdf = false;
    $columns = [];
    $title = "";

    if (!empty($postData['export_pdf']) && $postData['export_pdf']) {
      $exportPdf = true;
      $columns = [
        "Code",
        "Name",
        "Trading Area",
        "Part of EU Region"
      ];
      $title = "Country";
    }


    foreach ($records as $record) {
      if ($exportPdf) {
        $data[] = array(
          "CTY_IdCountry"       => $record->CTY_IdCountry,
          "CTY_NmeCountry"           => $record->CTY_NmeCountry,
          "CTY_IdTradingArea"            => $record->CTY_IdTradingArea,
          "CTY_IsPartOfEU"            => $record->CTY_IsPartOfEU,
        );
      } else {
        $action = '<a href="#" class="custom-edit menu-item-link" data-label="Edit Country" data-url="' . config("app.client_url") . '/countiries/edit/' . $record->CTY_IdCountry . '">Edit</a>';
        $action .= ' | <a href="#" class="custom-remove" data-role="delete-country" data-id="' . $record->CTY_IdCountry . '">Remove</a>';



        $data[] = array(

          "CTY_IdCountry"                => $record->CTY_IdCountry,
          "CTY_NmeCountry"                => $record->CTY_NmeCountry,
          "CTY_DtLastUpdated"                => $record->CTY_DtLastUpdated,
          "CTY_NmeUserLastUpdated"                => $record->CTY_NmeUserLastUpdated,
          "CTY_IsPartOfEU"                => $record->CTY_IsPartOfEU,
          "CTY_IdTradingArea"                => $record->CTY_IdTradingArea,
          "Action" => $action


        );
      }
    }

    if ($exportPdf) {
      $status   = "0";
      $message  = "";
      $errors   = [];
      $reportService = (new ReportService($data, $columns, $title));
      $fileLink = $reportService->handle();

      if ($fileLink) {
        $message = "Report generated successfully";
      } else {
        $message = "no data to show for your request";
      }
      return response()->json(['status' => $status, 'error' => (object)$errors, 'message' => $message, 'oData' => ['link' => $fileLink]], 200);
    }

    ## Response

    $response = array(

      "draw" => intval($draw),

      "iTotalRecords" => $totalRecords,

      "iTotalDisplayRecords" => $totalRecordwithFilter,

      "aaData" => $data

    );



    return $response;
  }
}
