<?php

namespace App\Services;

use App\Models\File;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Facades\View;
use Mpdf\Mpdf;

class ReportService
{
    public $rows, $columns, $title;

    public function __construct($rows, $columns, $title)
    {
        $this->rows = $rows;
        $this->columns = $columns;
        $this->title = $title;
    }

    public function handle()
    {

        $mpdf = new Mpdf();
        $html = View::make('report', [
            'rows' => $this->rows,
            'columns' => $this->columns,
            'title' => $this->title
        ]);
        $mpdf->WriteHTML($html);
        $extension = 'pdf';
        $directory = 'reports';
        $filename = 'report_' . time() . ".$extension";
        // Save PDF to storage
        if (!Storage::exists($directory)) {
            Storage::makeDirectory($directory, 0777, true, true);
        }
        $filePath = "$directory/$filename";
        Storage::disk('public')->put($filePath, $mpdf->Output('', 'S'));
        $file = File::create([
            'file_name' => $filename,
            'file_original_name' => $filename,
            'description' => "report in pdf",
            'path' => $filePath,
            'file_type' => 'pdf',
        ]);

        // $mpdf->Output($filePath, 'F');
        $downloadPath = Storage::disk('public')->url($filePath);
        return $downloadPath;
    }
}
