<?php

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Route;
use App\Http\Controllers\API\V1\AuthController;
use App\Http\Controllers\CompanyController;
use App\Http\Controllers\master\ReportController;

/*
|--------------------------------------------------------------------------
| API Routes
|--------------------------------------------------------------------------
|
| Here is where you can register API routes for your application. These
| routes are loaded by the RouteServiceProvider within a group which
| is assigned the "api" middleware group. Enjoy building your API!
|
*/

Route::middleware('jwt.auth')->get('/user', function (Request $request) {
    return $request->user();
});


Route::namespace('App\Http\Controllers\master')->middleware('jwt-auth')->name("api.")->group(function () {

    Route::post('/get-menu', 'SideBarController@get_menu');
    Route::post('/get-screen-data', 'SideBarController@getPages');
    Route::get('/get-companies', [CompanyController::class, 'index']);
    Route::post('/get-report', [ReportController::class, 'report'])->name('report');
    Route::post('/verify-login', [AuthController::class, 'verifyLogin'])->name('report');
    Route::post('/refresh-token', [AuthController::class, 'refreshToken'])->name('report');
    Route::post('/update-setting', [AuthController::class, 'updateSetting'])->name('report');


    // User related
    Route::post('/client-connected', [AuthController::class, 'clientConnected']);


    Route::post('/get-country-list', 'CountryController@index')->name('get-cuntry-list');
    Route::post('/get-country-data', 'CountryController@show')->name('get-cuntry-data');
    Route::post('/create-country-data', 'CountryController@create')->name('create-country-data');
    Route::post('/update-country-data', 'CountryController@update')->name('update-country-data');
    Route::post('/delete-country-data', 'CountryController@delete')->name('delete-country-data');
    Route::post('/datatable_list', 'CountryController@datatable_list')->name('datatable_list');

    Route::get('/get-trade', 'CountryController@get_trade')->name('get-trade');

    // Account category Routes
    Route::get('/get-sub-category-list', 'CategoryController@subCategory')->name('get-sub-category-list');
    Route::post('/get-category-list', 'CategoryController@index')->name('get-category-list');
    Route::post('/datatable_category_list', 'CategoryController@datatable_list')->name('datatable_category_list');
    Route::post('/create-category', 'CategoryController@store')->name('create_category');
    Route::post('/create-category-sub-category', 'CategoryController@storeCategory')->name('create-category-sub-category');

    Route::post('/update-category', 'CategoryController@update')->name('update_category');
    Route::get('/get-category/{id}', 'CategoryController@edit');
    Route::post('/delete-category', 'CategoryController@delete');


    // Unit of Measure Routes

    Route::post('/datatable_measure_list', 'MeasureController@datatable_list')->name('datatable_measure_list');
    Route::post('/create-measure', 'MeasureController@store')->name('create_measure');
    Route::get('/get-measure/{id}', 'MeasureController@edit');
    Route::post('/update-measure', 'MeasureController@update')->name('update_measure');
    Route::post('/delete-measure', 'MeasureController@delete');

    Route::get('/datatable_system_parameter_list', 'MeasureController@system_parameter')->name('datatable_system_parameter_list');
    Route::get('/uom-commodityuom/{id}', 'MeasureController@UOM_CommodityUOM')->name('uom-commodityuom');

    Route::post('/store-commodity-uom', 'MeasureController@insertOrUpdate_CONT_CommodityUOM')->name('store-commodity-uom');
});

Route::post('register', [AuthController::class, 'register']);
Route::post('login', [AuthController::class, 'login']);
Route::post('refresh_token_pinCode', [AuthController::class, 'refreshTokenPinCode']);
